#pragma once

/**
 * GenerationParameters.h - Bitcoin poscript generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */


#include <string>
#include "util.h"

class ExactAmount
{
private:

	ExactAmount(const std::string& inWhole, const std::string& inPart) : whole(inWhole), part(inPart) {}

public:

	ExactAmount() {}

	static ExactAmount fromStr(const std::string& str)
	{
		const int dotPos = str.find(".");
		if(dotPos == std::string::npos)
		{
			return ExactAmount(str, "");
		}
		else
		{
			const std::string wholeStr = str.substr(0, dotPos);
			const std::string partStr = str.substr(dotPos + 1);
			
			return ExactAmount(wholeStr, partStr);			
		}
		
	}

	bool zero() const
	{
		return (toDouble() == 0.0);
	}

	std::string zeroIfEmpty(const std::string& str) const
	{
		if(str.empty())
		{
			return "0";
		}
		else
		{
			return str;
		}
	}

	std::string toString() const
	{		
		return (zeroIfEmpty(whole) + "." + zeroIfEmpty(part));
	}

	std::string toOptionalString() const
	{
		if(zero())
		{
			return "";
		}
		else
		{
			return toString();
		}
	}

	double toDouble() const
	{
		const double val = std::stod(toString());
		return val;
	}


private:
	std::string whole;
	std::string part;
};






