

#include "OtpFolderHandler.h"

#include "currencySpecific.h"
#include "FileUtil.h"
#include "Directory.h"



std::string OtpFolderHandler::getPrivPadTotFolder(const std::string& folder, const int numPad)
{
	const std::string baseFolder = (folder + "/cold/privpad" + intToString(numPad));			
	const std::string totFolder = (baseFolder + "/" + getCurrencyShortName() + "/");
	
	return totFolder;
}


std::string OtpFolderHandler::getPublicTotFolder(const std::string& pubFolder)
{
	const std::string baseFolder = (pubFolder + "/cold/pub");		
	const std::string totFolder = (baseFolder + "/" + getCurrencyShortName() + "/");
	
	return totFolder;
}


std::string OtpFolderHandler::getPrivEmptyFile(const std::string& privFolder, const int padnum, const int addressNum)
{
	const std::string theFolder	= getTotalPrivFolder(privFolder, padnum, addressNum);

	if(!directoryExist(theFolder))
	{
		std::cout << "Folder does not exist: " << theFolder << std::endl;
		return "";
	}

	const std::string theFile = theFolder + "empty_" + getCurrencyShortName() + "_address_" + intToString(addressNum) + "_pad.txt";	

	return theFile;
}


EntroInfoSet OtpFolderHandler::getPublicOtpInfoList(const std::string& folder)
{	
	Directory directory(folder);
	
	const std::set<std::string> directories = directory.getEntries();
	
	EntroInfoSet infoSet;
	
	for(const std::string& dir : directories)
	{		
		if(!startsWith(dir, "pub_"))
		{
			continue;
		}
		
		const int num = getDirEntryNum(dir, "pub");

		const bool pubEmptyExists = OtpFolderHandler::pubEmptyFileExists(folder, num);
		
		const std::string entryFolder = folder + dir;

		const EntryInfo entryInfo = readPubEntryInfo(entryFolder, num, pubEmptyExists);
		infoSet.insert(std::pair<int, EntryInfo>(num, entryInfo));
	}

	return infoSet;
}




EntroInfoSet OtpFolderHandler::getPrivOtpInfoList(const std::string& folder, const int padnum)
{
	EntroInfoSet infoSet;
	
	Directory directory(folder);
	
	const std::set<std::string> directories = directory.getEntries();
	
	std::map<int, EntryInfo> info;
	
	for(const std::string& dir : directories)
	{
		if(!startsWith(dir, "priv_"))
		{
			continue;
		}
		
		const int num = getDirEntryNum(dir);
		
		const std::string entryFolder = folder + dir;
		const EntryInfo entryInfo = readEntryInfo(entryFolder, num, padnum);
		infoSet.insert(std::pair<int, EntryInfo>(num, entryInfo));
	}
	
	return infoSet;
}




EntryInfo OtpFolderHandler::readEntryInfo(const std::string& dir, const int num, const int padnum)
{
	const std::string filePath = dir + "/" + getCurrencyShortName() + "_address_" + intToString(num) + "_private_pad" + intToString(padnum) + ".txt";

	const std::string emptyFilePath = dir + "/" + "empty_" + getCurrencyShortName() + "_address_" + intToString(num) + "_pad.txt";

	bool isEmptyAmount(false);

	if(fileExist(emptyFilePath))
	{
		isEmptyAmount = true;
	}

	const std::string fileContent = readFile(filePath);

	const std::list<std::string> lines = splitLines(fileContent);

	std::string pad;
	std::string date;
	std::string accountName;

	for(const std::string& line : lines)
	{
		const std::pair<std::string, std::string> keyVal = splitColon(line);
		
		if(keyVal.first == "Private PAD")
		{
			pad = keyVal.second;
		}
		else if(keyVal.first == "Generation time")
		{
			date = keyVal.second;
		}
		else if(keyVal.first == "Account name")
		{
			accountName = keyVal.second;			
		}
	}

	ExactAmount dummyPrice;
	ExactAmount dummyAmount;

	
	const EntryInfo entry(num, date, accountName, pad, dummyPrice, dummyAmount, isEmptyAmount);

	return entry;
}


std::string OtpFolderHandler::getPubEmptyFile(const std::string& pubFolder, const int addressNum)
{
	const std::string theFolder	= getTotalPubFolder(pubFolder, addressNum);

	if(!directoryExist(theFolder))
	{
		std::cout << "Public folder does not exist: " << theFolder << std::endl;
		throw std::string("Public folder does not exist");
	}

	const std::string theFile = theFolder + "empty_" + getCurrencyShortName() + "_address_" + intToString(addressNum) + "_pad.txt";	

	return theFile;
}





int OtpFolderHandler::getDirEntryNum(const std::string& dir, const std::string prefix) // = "priv")
{
	const std::string start = prefix + "_" + getCurrencyShortName();
	const std::string folderStart = start + "_";

	if(!startsWith(dir, folderStart))	//Wrong prefix
	{
		return 0;
		
	}
	const std::string mystart = dir.substr(0, folderStart.size());
	
	const std::string num = dir.substr(folderStart.size());
	
	const int theNum = stoi(num);
	return theNum;
}


std::string OtpFolderHandler::removeWhitespaceEnd(const std::string& str)
{
	if(str.empty())
	{
		return "";
	}
	
	std::list<std::string> rows;
	int lastPos(0);
	for(int i = str.size() - 1 ; i >= 0 ; i--)
	{
		const unsigned char c = str[i];
		if(c != ' ')
		{
			return str.substr(0, i + 1);
		}
	}

	return str;
}


std::pair<std::string, std::string> OtpFolderHandler::splitColon(const std::string& line)
{
	const int pos = line.find(": ");
	if(pos == std::string::npos)
	{
		return std::pair<std::string, std::string>("", "");
	}
	
	const std::string key = line.substr(0, pos);
	const std::string keyOnly = removeWhitespaceEnd(key);
	
	const std::string value = line.substr(pos + 2);
	
	return std::pair<std::string, std::string>(keyOnly, value);
}


std::string OtpFolderHandler::getTheFilename(const std::set<std::string>& directories, const int num)
{	
	const std::string start = getCurrencyShortName() + "_address_" + intToString(num) + "_pad";
	const std::string end = "_share.txt";
	
	for(const std::string& dir : directories)
	{		
		if(!startsWith(dir, start))
		{
			continue;
		}

		if(!endsWith(dir, end))
		{
			continue;
		}
		
		return dir;	//Ok, found match
	}
	
	throw std::string("Did not find public file");
	return "";
}



EntryInfo OtpFolderHandler::readPubEntryInfo(const std::string& baseDir, const int num, const bool isEmptyAmount)
{	
	Directory directory(baseDir);
	
	const std::set<std::string> directories = directory.getEntries();
	
	const std::string theTxtFilename = getTheFilename(directories, num);
	
	const std::string totPath = baseDir + "/" + theTxtFilename;

	const std::string fileContent = readFile(totPath);

	const std::list<std::string> lines = splitLines(fileContent);

	std::string address;
	std::string date;
	std::string accountName;
	ExactAmount amount;
	ExactAmount price;

	for(const std::string& line : lines)
	{
		const std::pair<std::string, std::string> keyVal = splitColon(line);
		
		if(keyVal.first == "Public address")
		{
			address = keyVal.second;
		}
		else if(keyVal.first == "Cash address")
		{
			address = keyVal.second;
		}		
		else if(keyVal.first == "Generation time")
		{
			date = keyVal.second;
		}
		else if(keyVal.first == "Account name")
		{
			accountName = keyVal.second;
		}
		else if(keyVal.first == "Amount")
		{
			const std::string amountStr = keyVal.second;
			amount = ExactAmount::fromStr(amountStr);
		}		
		else if(keyVal.first == "Price")
		{
			const std::string priceStr = keyVal.second;
			price = ExactAmount::fromStr(priceStr);
		}
	}


	const EntryInfo entry(0, date, accountName, address, price, amount, isEmptyAmount);

	return entry;
}





std::string OtpFolderHandler::getTotalPubFolder(const std::string& pubFolder, const int addressNum)
{
	const std::string baseFolder = (pubFolder + "/cold/pub");		
	const std::string totFolder = (baseFolder + "/" + getCurrencyShortName() + "/");
	
	const std::string theFolder	= totFolder + "pub_" + getCurrencyShortName() + "_" + intToString(addressNum) + "/";

	return theFolder;
}



std::string OtpFolderHandler::getTotalPrivFolder(const std::string& pubFolder, const int padnum, const int addressNum)
{
	const std::string baseFolder = (pubFolder + "/cold/privpad");		
	const std::string totFolder = (baseFolder + intToString(padnum) + "/" + getCurrencyShortName() + "/");
	
	const std::string theFolder	= totFolder + "priv_" + getCurrencyShortName() + "_" + intToString(addressNum);

	return theFolder;
}






std::string OtpFolderHandler::getMyPubFolder(const std::string& pubFolder, const int addressNum)
{	
	const std::string theFolder	= pubFolder + "pub_" + getCurrencyShortName() + "_" + intToString(addressNum) + "/";

	return theFolder;
}


bool OtpFolderHandler::pubEmptyFileExists(const std::string& pubFolder, const int addressNum)
{
	//std::cout << "Checking if pub folder exist: " << pubFolder << ": " << addressNum << std::endl;
	
	const std::string theFolder	= OtpFolderHandler::getMyPubFolder(pubFolder, addressNum);

	if(!directoryExist(theFolder))
	{
		std::cout << "THE Folder does not exist: " << theFolder << std::endl;
		
		return false;
		//throw std::string("Folder does not exist");
	}

	const std::string theFile = theFolder + "empty_" + getCurrencyShortName() + "_address_" + intToString(addressNum) + "_pad.txt";	

	if(!fileExist(theFile))
	{
		//std::cout << "THE file does not exist: " << theFile << std::endl;
		
		return false;
	}

	return true; //theFile;
}


	








