#pragma once

/**
 * postscriptGeneration.h - Bitcoin poscript generator
 *
 * Copyright (C)2023 The bitgen developers
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */


#include "ExactAmount.h"

#include <map>

class EntryInfo
{
public:
	EntryInfo(int inNum, const std::string& inDate, const std::string& inAccountName, const std::string& inPad, const ExactAmount& inPrice, const ExactAmount& inAmount, const bool inIsEmptyAmount = false) : 
		num(inNum), date(inDate), accountName(inAccountName), pad(inPad), price(inPrice), amount(inAmount), isEmptyAmount(inIsEmptyAmount) {}

	int num;
	std::string date;
	std::string accountName;
	std::string pad;
	ExactAmount price;	
	ExactAmount amount;
	bool isEmptyAmount;
};


class EntroInfoSet
{
public:

int getLastNum() const
{
	int last(0);
		
	for(const std::pair<int, EntryInfo>& val : info)
	{
		last = val.first;
	}

	return last;
}


int numNonempty() const
{
	int num(0);
	for(const std::pair<int, EntryInfo>& val : info)
	{
		if(val.second.isEmptyAmount)
		{
			continue;
		}
		
		num++;
	}	

	return num;
}

	void insert(const std::pair<int, EntryInfo>& val)
	{
		info.insert(val);
	}


	double sum() const
	{
		double sum(0.0);
		for(const std::pair<int, EntryInfo>& val : info) 
		{
			sum += val.second.amount.toDouble();
		}
		
		return sum;
	}

	double priceSum() const
	{
		double sum(0.0);
		for(const std::pair<int, EntryInfo>& val : info) 
		{
			sum += val.second.price.toDouble();
		}
		
		return sum;
	}



	int size() const
	{
		return info.size();
	}

	const std::map<int, EntryInfo>& getRead() const { return info; }

private:
	std::map<int, EntryInfo> info;
};

