#pragma once


class QRData
{
public:
	QRData(const QRInfo& inQrinfo, const std::string inStr1, const std::string& inStr2) : 
		qrinfo(inQrinfo), str1(inStr1), str2(inStr2) {}

	const QRInfo qrinfo;
	const std::string str1;
	const std::string str2;
};



class PSTextWriter
{
	public:
	
	PSTextWriter(const bool secondPartPrivate, const int inMargin, const int inColumnTwoPos = 80) : 
		margin(inMargin), lineNum(secondPartPrivate ? 19 : 2), columnTwoPos(inColumnTwoPos) {}
	

	void write(const std::string& text1, const std::string& text2)
	{
		ret += PostscriptRawGenerator::textAt(margin,       PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), text1);		
		ret += PostscriptRawGenerator::textAt(margin + columnTwoPos, PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), text2);				
	}
	

	void writeSec2(const std::string& text1, const std::string& text2)
	{
		ret += PostscriptRawGenerator::textAt(margin + 250,       PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), text1);		
		ret += PostscriptRawGenerator::textAt(margin + 250 + columnTwoPos, PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), text2);				
	}
	
	void writeLn(const std::string& text)
	{
		ret += PostscriptRawGenerator::textAt(margin,       PostscriptLineGenerator::publicLinenoNumToYpos(lineNum++), text);		
	}

	void writeItem(const std::string& seqNum, const std::string& num, const std::string& a, const std::string& b, const std::string& c)
	{
		ret += PostscriptRawGenerator::textAt(margin,     PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), num);
		ret += PostscriptRawGenerator::textAt(margin + 50,     PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), a);
		
		ret += PostscriptRawGenerator::textAt(margin + 250,       PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), b);

		addPs("Tiny_font setfont\n");		
		ret += PostscriptRawGenerator::textAt(margin + 25,     PostscriptLineGenerator::publicLinenoNumToYpos(lineNum) - 12, seqNum);
		addPs("Small_font setfont\n");
		
		ret += PostscriptRawGenerator::textAt(margin + 50,       PostscriptLineGenerator::publicLinenoNumToYpos(lineNum++) - 12, c);
		lineNum++;
	}

	void writePubItem(const std::string& seqNum, const std::string& num, const std::string& a, const std::string& b, const std::string& amount, const std::string& price, const std::string& c)
	{
		ret += PostscriptRawGenerator::textAt(margin,     PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), num);
		ret += PostscriptRawGenerator::textAt(margin + 50,     PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), a);
		
		ret += PostscriptRawGenerator::textAt(margin + 250,       PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), b);				
		ret += PostscriptRawGenerator::textAt(margin + 400,       PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), amount);
		ret += PostscriptRawGenerator::textAt(margin + 460,       PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), price);

		addPs("Tiny_font setfont\n");
		
		ret += PostscriptRawGenerator::textAt(margin + 25,     PostscriptLineGenerator::publicLinenoNumToYpos(lineNum) - 12, seqNum);
		ret += PostscriptRawGenerator::textAt(margin + 50,       PostscriptLineGenerator::publicLinenoNumToYpos(lineNum++) - 12, c);
		lineNum++;

		addPs("Small_font setfont\n");
	}


	void newPage()
	{
		ret += PostscriptRawGenerator::endPage(pageNum);
		pageNum++;
		
		ret += PostscriptRawGenerator::pageIntro(pageNum);
		lineNum = 2;
	}



	//void writeLn(const std::string& text)
	//{
	//	ret += PostscriptGenerator::textAt(margin,       publicLinenoNumToYpos(lineNum++) /* 740 */, text);		
	//}

	void writeLn(const std::string& text1, const std::string& text2)
	{
		ret += PostscriptRawGenerator::textAt(margin,       PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), text1);		
		ret += PostscriptRawGenerator::textAt(margin + columnTwoPos, PostscriptLineGenerator::publicLinenoNumToYpos(lineNum++), text2);				
	}

	void write(const std::string& text, const int columnPos)
	{
		ret += PostscriptRawGenerator::textAt(margin + columnPos, PostscriptLineGenerator::publicLinenoNumToYpos(lineNum), text);				
	}

	int getYPos() const
	{
		return PostscriptLineGenerator::publicLinenoNumToYpos(lineNum);
	}


	void addPs(const std::string& ps)
	{
		ret += ps;
	}

	void newLine()
	{
		lineNum++;
	}

	void addNewline(const int numNewline)
	{
		lineNum += numNewline;
	}


	void addQR(const QRInfo& qrinfo, const std::string& str1, const std::string& str2, const double val, const bool columnTwo = false) //int xOffset = 0)
	{
		const int xOffset = columnTwo ? 200 : 0;
		ret += PostscriptRawGenerator::generateQRPostscriptWithText(qrinfo, str1, str2, margin + xOffset, PostscriptLineGenerator::publicLinenoNumToYpos(lineNum) , val);
	}



	void addQR(const QRData& qrinfo, const double val, const bool columnTwo = false) 
	{
		const int xOffset = columnTwo ? 200 : 0;
		ret += PostscriptRawGenerator::generateQRPostscriptWithText(qrinfo.qrinfo, qrinfo.str1, qrinfo.str2, margin + xOffset, PostscriptLineGenerator::publicLinenoNumToYpos(lineNum) , val);
	}


	std::string getPs() const { return ret; }
	
	int getLineNum() const { return lineNum; }
	
	void setLine(const int line)
	{
		lineNum = line;
	}
	
	void box(const int xPos, const int sideLen)
	{
		std::list<Coordinate> coordinates;
		const int yPos = getYPos();
		
		const Coordinate coordinate1(xPos          , yPos);
		const Coordinate coordinate2(xPos          , yPos + sideLen);
		const Coordinate coordinate3(xPos + sideLen, yPos + sideLen);
		const Coordinate coordinate4(xPos + sideLen, yPos);
		const Coordinate coordinate5(xPos          , yPos);
		
		coordinates.push_back(coordinate1);
		coordinates.push_back(coordinate2);
		coordinates.push_back(coordinate3);
		coordinates.push_back(coordinate4);
		coordinates.push_back(coordinate5);
		
		ret += PostscriptLineGenerator::lineBetweenPoints(coordinates);	
	}
	

	void writeTab(const int i, const std::string& word, const int yOffset = 0)
	{
		const int tabSize(0);		
		ret += PostscriptRawGenerator::textAt(margin + tabSize + (i * 63), PostscriptLineGenerator::publicLinenoNumToYpos(lineNum) + yOffset, word);	
	}

	
	void endPage()
	{
		ret += PostscriptRawGenerator::endPage(getPageNum());
	}
	
	
	int getPageNum() const
	{
		return pageNum;
	}
		

	void endFile()
	{
		ret += "%%EOF\n";
	}
		

	void introBeginPage(const std::string& title)
	{
		ret += PostscriptRawGenerator::intro(title);	
		ret += PostscriptRawGenerator::beginPage();
	}
		
		
private:
	int margin;
	int columnTwoPos = 100;
	int pageNum = 1;
	int lineNum = 0;
	std::string ret;
};




