/**
 * txtGeneration.cpp - Bitcoin address generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
/**
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE. 
*/


#include "txtGeneration.h"
#include "util.h"
#include "FileUtil.h"
#include "currencySpecific.h"
#include "bigintutil.h"
#include "hash.h"
#include "StaticXor.h"

#include <iostream>










FilePair generateExtendedTxtFiles(const std::string& pub, const std::string priv, 
					const std::string& genMethod, 
					const std::string& thetime, const std::string& accountName,
					const std::string& versionNumber, const std::string& applicationName,
					const std::list<ExtraInfo>& generationExtra,
					const std::string& derivationPath,
					const bool currencySpecific
					)
{
	
	std::string pubTxt;

	if(currencySpecific)
	{
		pubTxt += "Currency         : ";
		const std::string currencyName = getCurrencyName();	
		pubTxt += currencyName;
		pubTxt += "\n";
	}
	
	pubTxt += "Public           : ";
	pubTxt += pub;
	pubTxt += "\n";
	
	pubTxt += "Generated by     : ";
	pubTxt += applicationName;
	pubTxt += " ";
	pubTxt += versionNumber;
	pubTxt += "\n";
	
	pubTxt += "Generation time  : ";
	pubTxt += thetime;
	pubTxt += "\n";
	if(!accountName.empty())
	{
		pubTxt += "Account name     : ";
		pubTxt += accountName;
		pubTxt += "\n";	
	}

	pubTxt += "Derivation path  : ";
	pubTxt += derivationPath;
	pubTxt += "\n";
	
	/*
	if(!params.getNodeName().empty())
	{
		pubTxt += minSize("TheName ", 17);
		
		pubTxt += ": ";
		pubTxt += params.getNodeName();
		pubTxt += "\n";		
	}	
	*/
	std::string privTxt(pubTxt);
	
	privTxt += "Private          : ";
	privTxt += priv;
	privTxt += "\n";
	
	privTxt += "Generation method: ";
	privTxt += genMethod;
	privTxt += "\n";

	//privTxt += "Mnemonic         : ";
	//privTxt += mnemonic;
	//privTxt += "\n";

// minSize(std::string str, const int minSize, const bool right = false);


	for(std::list<ExtraInfo>::const_iterator it = generationExtra.begin() ;
		it != generationExtra.end() ;
		++it)
	{
		const ExtraInfo& extra = (*it);
		privTxt += minSize(extra.title, 17);
		privTxt += ": ";
		privTxt += extra.value;
		privTxt += "\n";
		
	}

	//if(!generationExtra.empty())
	//{
	//	privTxt += "\n";
	//}


	FilePair fpair(pubTxt, privTxt);
	return fpair;
}






//info.params.getGenerateExtended()
					//const std::string& genMethod, 
					//const std::list<ExtraInfo>& generationExtra,
					//const std::string& thetime, 
					//const std::string& accountName,
					//const std::string& versionNumber, 
					//const std::string& applicationName,
					//const bool includedExtendedAddress

FilePair generateMnemonicFile(const GenerationInfo& info,
					const Mnemonic& mnemonic,
					const bool currencySpecific,
					const std::string& xpriv, 
					const std::string& xpub,
					const std::string& derivedPubStrWithExtraInfo,
					const std::string& derivedXpub
					)
{
	
	std::string pubTxt;

	if(currencySpecific || !derivedXpub.empty())
	{
		pubTxt += "Currency         : ";
		const std::string currencyName = getCurrencyName();	
		pubTxt += currencyName;
		pubTxt += "\n";
	}
		
	pubTxt += "Generated by     : ";
	pubTxt += info.getApplicationName(); //applicationName;
	pubTxt += " ";
	pubTxt += info.getVersionNumber(); //versionNumber;
	pubTxt += "\n";
	
	pubTxt += "Generation time  : ";
	pubTxt += info.getTime(); //thetime;
	pubTxt += "\n";
	if(!info.getAccountName().empty())
	{
		pubTxt += "Account name     : ";
		pubTxt += info.getAccountName(); //accountName;
		pubTxt += "\n";	
	}

	if(!info.getNodeName().empty())
	{
		pubTxt += minSize("NodeName ", 17);
		
		pubTxt += ": ";
		pubTxt += info.getNodeName();
		pubTxt += "\n";
		
	}


	if(info.params.getGenerateExtended())
	{
		pubTxt += "xpub             : ";
		pubTxt += xpub;
		pubTxt += "\n";

		if(!derivedXpub.empty())
		{
			
			pubTxt += "derived orig xpub: ";
			pubTxt += derivedPubStrWithExtraInfo;
			pubTxt += "\n";
			
			pubTxt += "derived xpub     : ";
			pubTxt += derivedXpub;
			pubTxt += "\n";
		}
	}

	pubTxt += "Num words        : ";
	pubTxt += intToString(mnemonic.numberWords());
	pubTxt += "\n";

/*	
	if(!params.getNodeName().empty())
	{
		pubTxt += minSize("TheName ", 17);
		
		pubTxt += ": ";
		pubTxt += params.getNodeName();
		pubTxt += "\n";		
	}	
*/
	std::string privTxt(pubTxt);
	
	privTxt += "Generation method: ";
	privTxt += info.getGenerator(); //genMethod;
	privTxt += "\n";

	privTxt += "Mnemonic         : ";
	privTxt += mnemonic.toStr();
	privTxt += "\n";

	if(info.params.getGenerateExtended())
	{
		privTxt += "xpriv            : ";
		privTxt += xpriv;
		privTxt += "\n";
	}


	for(std::list<ExtraInfo>::const_iterator it = info.getGenerationExtra().begin() ;
		it != info.getGenerationExtra().end() ;
		++it)
	{
		const ExtraInfo& extra = (*it);
		privTxt += minSize(extra.title, 17);
		privTxt += ": ";
		privTxt += extra.value;
		privTxt += "\n";		
	}

	const FilePair filePair(pubTxt, privTxt);


	return filePair; //privTxt;
}





std::string generateStaticXorFile(const std::string& thetime, 
	const std::string& versionNumber, const std::string& applicationName, 
	const std::string& accountName,
	const BigInt<256>& staticXor, 
	const std::string& nodeName)
{
	std::string pubTxt;
		
	pubTxt += "Generated by     : ";
	pubTxt += applicationName;
	pubTxt += " ";
	pubTxt += versionNumber;
	pubTxt += "\n";
	
	pubTxt += "Generation time  : ";
	pubTxt += thetime;
	pubTxt += "\n";
	
	if(!accountName.empty())
	{
		pubTxt += "Account name     : ";
		pubTxt += accountName;
		pubTxt += "\n";	
	}

	pubTxt += "Node name        : ";
	pubTxt += nodeName;
	pubTxt += "\n";

	
	std::string privTxt(pubTxt);

	const std::string staticXorStr = hexString(staticXor.getBinaryLong());	
	
	privTxt += "Static XOR       : ";
	privTxt += staticXorStr;
	privTxt += "\n";
	
	const std::string hash = staticXorHash(staticXor);
	

	privTxt += "Hash             : ";
	privTxt += hexString(hash);
	privTxt += "\n";


	return privTxt;
}




const std::string generatePadTxtFile(
	const std::string& pad, const int padNum, const int totalPads,
	const std::string& genMethod, const std::string& thetime, 
	const std::string& versionNumber, const std::string& applicationName, const std::list<ExtraInfo>& generationExtra,
	const std::string& accountName,
	const BigInt<256>& staticXor, const int newNum)
{
	std::string pubTxt;

	pubTxt += "Currency         : ";
	const std::string currencyName = getCurrencyName();	
	pubTxt += currencyName;
	pubTxt += "\n";
	
	//pubTxt += "Public address   : ";
	//pubTxt += pubkey.toString();
	//pubTxt += "\n";
	
	//if(getCurrencyShortName() == "bch")
	//{
	//	pubTxt += "Public cashaddr  : ";
	//	pubTxt += pubkey.toCashAddrString();
	//	pubTxt += "\n";		
	//}
	
	pubTxt += "Generated by     : ";
	pubTxt += applicationName;
	pubTxt += " ";
	pubTxt += versionNumber;
	pubTxt += "\n";
	
	pubTxt += "Generation time  : ";
	pubTxt += thetime;
	pubTxt += "\n";
	
	if(newNum != 0)
	{
		pubTxt += "Number           : ";
		pubTxt += intToString(newNum);
		pubTxt += "\n";
	}
	
	if(!accountName.empty())
	{
		pubTxt += "Account name     : ";
		pubTxt += accountName;
		pubTxt += "\n";	
	}

	
	std::string privTxt(pubTxt);
	
	privTxt += "Private PAD      : ";
	privTxt += hexString(pad);
	privTxt += "\n";
	
	privTxt += "Pad num          : ";
	privTxt += intToString(padNum + 1);
	privTxt += "\n";
	
	privTxt += "Total pads       : ";
	privTxt += intToString(totalPads);
	privTxt += "\n";
	
	const BigInt<256> zero;
	if(staticXor != zero)
	{
		privTxt += "Static XOR hash  : ";
		const std::string hashPart = staticXorHash(staticXor);

		//const std::string hash = doubleSha256(staticXor.getBinaryLong());
		//const std::string hashPart = hash.substr(0, 4);
		privTxt += hexString(hashPart);
		privTxt += "\n";		
	}
	
	privTxt += "Generation method: ";
	privTxt += genMethod;
	if(staticXor != zero)
	{
		privTxt += " with static XOR";
	}
	privTxt += "\n";

	//privTxt += "Mnemonic         : ";
	//privTxt += mnemonic;
	//privTxt += "\n";



	for(std::list<ExtraInfo>::const_iterator it = generationExtra.begin() ;
		it != generationExtra.end() ;
		++it)
	{
		const ExtraInfo& extra = (*it);
		privTxt += minSize(extra.title, 17);
		
		privTxt += ": ";
		privTxt += extra.value;
		privTxt += "\n";
		
	}

	//if(!generationExtra.empty())
	//{
	//	privTxt += "\n";
	//}


	//FilePair fpair(pubTxt, privTxt);
	return privTxt;
}




FilePair generateTxtFiles(const AddressGenerationInfo& allInfo)
{
	
	std::string pubTxt;

	pubTxt += "Currency         : ";
	const std::string currencyName = getCurrencyName();	
	pubTxt += currencyName;
	pubTxt += "\n";

	if(getCurrencyShortName() != "bch_slp")
	{
		pubTxt += "Public address   : ";
		pubTxt += allInfo.pubaddr;
		pubTxt += "\n";
	}
	
	
	//if(getCurrencyShortName() == "bch" || getCurrencyShortName() == "bch_slp")
	//{
	//	pubTxt += "Cash address     : ";
	//	pubTxt += allInfo.getCashAddress();
	//	pubTxt += "\n";
	//
	//	pubTxt += "Cash address read: ";
	//	pubTxt += allInfo.getCashAddressRead();
	//	pubTxt += "\n";				
	//}
	
	pubTxt += "Generated by     : ";
	pubTxt += allInfo.getApplicationName();
	pubTxt += " ";
	pubTxt += allInfo.getVersionNumber();
	pubTxt += "\n";
	
	pubTxt += "Generation time  : ";
	pubTxt += allInfo.getTime();
	pubTxt += "\n";
	
	if(allInfo.getNewNum() != 0)
	{
		pubTxt += "Number           : ";
		pubTxt += intToString(allInfo.getNewNum());
		pubTxt += "\n";
	}
	
	if(!allInfo.getAccountName().empty())
	{
		pubTxt += "Account name     : ";
		pubTxt += allInfo.getAccountName();
		pubTxt += "\n";	
	}


	if(!allInfo.getNodeName().empty())
	{
		pubTxt += minSize("NodeName ", 17);
		
		pubTxt += ": ";
		pubTxt += allInfo.getNodeName();
		pubTxt += "\n";
		
	}
	
	if(!allInfo.getAmount().zero())
	{
		pubTxt += minSize("Amount ", 17);
		
		pubTxt += ": ";
		pubTxt += allInfo.getAmount().toString();
		pubTxt += "\n";
		
	}

	if(!allInfo.getPrice().zero())
	{
		pubTxt += minSize("Price ", 17);
		
		pubTxt += ": ";
		pubTxt += allInfo.getPrice().toString();
		pubTxt += "\n";
		
	}


	std::string privTxt(pubTxt);
	
	if(!allInfo.privaddr.empty())
	{
		privTxt += minSize(allInfo.privaddrName, 17) +  ": ";
		privTxt += allInfo.privaddr;
		privTxt += "\n";
	}
	
	//if(getCurrencyShortName() != "xmr" && getCurrencyShortName() != "zec")
	//{	
	//	privTxt += "Compressed       : ";
	//	privTxt += (allInfo.params.getCompressed() ? "yes" : "no");
	//	privTxt += "\n";
	//}
	
	//if(!accountName.empty())
	//{
	//	privTxt += "Account name     : ";
	//	privTxt += accountName;
	//	privTxt += "\n";	
	//}
	privTxt += "Generation method: ";
	privTxt += allInfo.getGenerator();
	const BigInt<256> zero;
	if(allInfo.getStaticXor() != zero)
	{
		privTxt += " with static xor";
	}
	privTxt += "\n";

	if(allInfo.getStaticXor() != zero)
	{
		privTxt += "Static xor hash  : ";
		const std::string xorHash = staticXorHash(allInfo.getStaticXor());
		privTxt += hexString(xorHash);
		privTxt += "\n";
	}


	if(!allInfo.getMnemonic().empty())
	{
		privTxt += "Mnemonic         : ";
		privTxt += allInfo.getMnemonic();
		privTxt += "\n";
	}

	const std::list<ExtraInfo>& generationExtra = allInfo.getGenerationExtra();


	for(std::list<ExtraInfo>::const_iterator it = generationExtra.begin() ;
		it != generationExtra.end() ;
		++it)
	{
		const ExtraInfo& extra = (*it);
		privTxt += minSize(extra.title, 17);
		
		privTxt += ": ";
		privTxt += extra.value;
		privTxt += "\n";
		
	}

	//if(!generationExtra.empty())
	//{
	//	privTxt += "\n";
	//}


	FilePair fpair(pubTxt, privTxt);
	return fpair;
}




void saveExtendedTxtFiles(const AddressGenerationInfo& allInfo, FilePair& txtFiles, const std::string& fileId, const bool currencySpecific, const bool isElectrum)
{	
	std::string currencySpecStr;
	if(currencySpecific || isElectrum)
	{
		currencySpecStr += getCurrencyShortName();
		currencySpecStr += "_";		
	}
	
	const std::string start = isElectrum ? "electrum" : "extended";
	
	const std::string numStr = (allInfo.getNewNum() == 0) ? "" : (intToString(allInfo.getNewNum()) + "_");

	
	const std::string pubFilename = start + "_" + currencySpecStr + numStr +  fileId.substr(0, 8) + "_share.txt";
	const std::string privFilename = start + "_" + currencySpecStr + numStr  + fileId.substr(0, 8) + "_private.txt";

	//const std::string fullPrivPath = (privPath == ".") ? privFilename : (privPath + "/" + privFilename);
	std::cout << "Saving priv txt: " << privFilename << std::endl;
	fileWrite(privFilename, txtFiles.privateFile);	

	//const std::string fullPubPath = (pubPath == ".") ? pubFilename : (pubPath + "/" + pubFilename);
	std::cout << "Saving pub txt : " << pubFilename << std::endl;
	fileWrite(pubFilename, txtFiles.publicFile);	
}




					//const std::string& mnemonic, 
					
void saveOtpFile(const std::string& not_used_pubkey, const std::string pad, const int indexPad, const int totalPads,
					const std::string& thetime, const std::string privPath,
					const std::string& versionNumber, const std::string& applicationName,
					const std::list<ExtraInfo>& generationExtra, const std::string& accountName,
					const std::string& generator,
					const BigInt<256>& staticXor, const int newNum,
					const std::string& privFilename)
{
		
	//std::cout << "In saveOtpFile" << std::endl;
	const std::string padFileContent = generatePadTxtFile(/*pubkey,*/ pad, indexPad, totalPads, generator, thetime, 
		versionNumber, applicationName, generationExtra, accountName, staticXor, newNum);
		
	const int numPad = indexPad + 1;
	
	//const std::string privFilename = /*pubkey.toString()*/ getCurrencyShortName() + "_address_"+ intToString(newNum)+ "_private_pad" + intToString(numPad) + ".txt";

	const std::string fullPrivPath = (privPath == ".") ? privFilename : (privPath + "/" + privFilename);
	std::cout << "Saving otp txt : " << withoutLeadingDot(fullPrivPath) << std::endl;
	fileWrite(fullPrivPath, padFileContent);	
}



					//const std::string& thetime, 
					//const std::string& accountName,
					//const std::string& versionNumber, const std::string& applicationName)

void savePubOtpTxtFiles(const std::string& filenameBase, 
					const std::string& pubkey, 
					const std::string pubPath, 					
					const GenerationInfo& info)					
{
	//std::cout << "IN savePubOtpTxtFiles: " << staticXor << std::endl;
	
	const std::string privateb58("WIFDUMMY");
	const std::string genMethod("methoddummy");
	const BigInt<256> staticXorDummy;
	const std::list<ExtraInfo> generationExtraDummy;
	const std::string mnemonicDummy;
	const bool compressedDummy(true);
	
	
	AddressGenerationInfo allInfo(pubkey, privateb58, "Wif", info);
	
	FilePair txtFiles = generateTxtFiles(allInfo);
					//pubkey, "FullCashAddrString", "CashAddrReadableString",	//Todo
					//privateb58, genMethod, generationExtraDummy,
					//thetime, accountName, mnemonicDummy, compressedDummy,
					//versionNumber, applicationName, staticXorDummy);
				
	const std::string pubFilename = filenameBase + "_" + pubkey.substr(0, 8) + "_share.txt";
		
	const std::string fullPubPath = (pubPath == ".") ? pubFilename : (pubPath + "/" + pubFilename);
	std::cout << "Saving pub txt : " << fullPubPath << std::endl;
	fileWrite(fullPubPath, txtFiles.publicFile);	
}



//const int newNum, const std::string& pubkey, const std::string& pubkeyFull, const std::string& pubkeyRead, 
//					const std::string privateb58, 
//					const std::string& genMethod, const std::list<ExtraInfo>& generationExtra,
//					const std::string& thetime, const std::string& accountName,
//					const std::string& mnemonic, const bool compressed, const std::string pubPath, const std::string privPath,
//					const std::string& versionNumber, const std::string& applicationName,
//					const bool isPad, const BigInt<256> staticXor
//					)
void saveTxtFiles(const std::string pubPath, const std::string privPath, const AddressGenerationInfo& allInfo)	
{
	//std::cout << "IN saveTxtFiles: " << staticXor << std::endl;
	
	FilePair txtFiles = generateTxtFiles(allInfo);
					//pubkey,  pubkeyFull, pubkeyRead,
					//privateb58, genMethod, generationExtra,
					//thetime, accountName, mnemonic, compressed,
					//versionNumber, applicationName, staticXor);
		

	const std::string padExtra = (allInfo.getIsPad() ? "_pad" : "");
	
	const std::string addressStr = allInfo.pubaddr;
	
	const std::string numStr = (allInfo.getNewNum() == 0) ? "" : (intToString(allInfo.getNewNum()) + "_");
	
	const std::string filenameStart = getCurrencyShortName() + "_address_" + numStr + addressStr.substr(0, 8) + padExtra;
	
	const std::string pubFilename = filenameStart + padExtra + "_share.txt";
	const std::string privFilename = filenameStart + padExtra + "_private.txt";

	if(!allInfo.getIsPad())
	{
		const std::string fullPrivPath = (privPath == ".") ? privFilename : (privPath + "/" + privFilename);
		std::cout << "Saving priv txt: " << fullPrivPath << std::endl;
		fileWrite(fullPrivPath, txtFiles.privateFile);	
	}
	
	
	const std::string fullPubPath = (pubPath == ".") ? pubFilename : (pubPath + "/" + pubFilename);
	std::cout << "Saving pub txt : " << fullPubPath << std::endl;
	fileWrite(fullPubPath, txtFiles.publicFile);	
}








TxtInfoFile getTxtFiles(const std::string& pubkey, const std::string privateb58, 
					const std::string& genMethod, const std::list<std::string>& generationExtra,
					const std::string& thetime, const std::string& accountName,
					const std::string& mnemonic, const bool compressed,
					const std::string& versionNumber, const std::string& applicationName)
{
	std::string pubTxt;

	pubTxt += "Currency         : ";
	
	const std::string currencyName = getCurrencyName();	
	pubTxt += currencyName;

	pubTxt += "\n";
	
	pubTxt += "Public address   : ";
	pubTxt += pubkey;
	pubTxt += "\n";
	
	pubTxt += "Generated by     : ";
	pubTxt += applicationName;
	pubTxt += " ";	
	pubTxt += versionNumber;
	pubTxt += "\n";
	
	pubTxt += "Generation time  : ";
	pubTxt += thetime;
	pubTxt += "\n";
	if(!accountName.empty())
	{
		pubTxt += "Account name     : ";
		pubTxt += accountName;
		pubTxt += "\n";	
	}

	
	//const std::string pubFilename = pubkey + "_share.txt";
	//std::cout << "Saving pub txt : " << pubFilename << std::endl;
	//fileWrite(pubFilename, pubTxt);	
	
	
	
	std::string privTxt(pubTxt);
	
	privTxt += "Private key (WIF): ";
	privTxt += privateb58;
	privTxt += "\n";
	//privTxt += "Public address   : ";
	//privTxt += pubkey;
	//privTxt += "\n";
	//privTxt += "Generation time  : ";
	//privTxt += thetime;
	//privTxt += "\n";
	privTxt += "Compressed       : ";
	privTxt += (compressed ? "yes" : "no");
	privTxt += "\n";
	
	if(!accountName.empty())
	{
		privTxt += "Account name     : ";
		privTxt += accountName;
		privTxt += "\n";	
	}
	privTxt += "Generation method: ";
	privTxt += genMethod;
	privTxt += "\n";

	privTxt += "Mnemonic         : ";
	privTxt += mnemonic;
	privTxt += "\n";



	for(std::list<std::string>::const_iterator it = generationExtra.begin() ;
		it != generationExtra.end() ;
		++it)
	{
		const std::string& extra = (*it);
		privTxt += extra;
	}

	if(!generationExtra.empty())
	{
		privTxt += "\n";
	}


	const TxtInfoFile fileInfo(pubTxt, privTxt);
	return fileInfo;
	
	//const std::string privFilename = pubkey + "_private.txt";
	//std::cout << "Saving priv txt: " << privFilename << std::endl;
	//fileWrite(privFilename, privTxt);	

}



