#ifndef BITGEN_CCONFIG_H
#define BITGEN_CCONFIG_H

/**
 * bitgenConfig.h - Bitcoin poscript generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <iostream>
#include <stdlib.h>
#include <list>
#include <vector>

#include "bigint.h"
#include "bigintutil.h"



std::string getNodeName();



class BitgenConfig
{
public:
	static int getConfiguredNum(const bool increase = true);
	static void setConfiguredNum(const int val);

	static void saveStaticXor(const BigInt<256>& privkey);
	static BigInt<256> getStaticXor(const bool useStaticXorIfPresent = true);
	static void deleteStaticXor();


	static void configureLang(const std::string& lang);
	static std::string getConfiguredLang();

	static void addFoldersToConfig(const std::list<std::string>& folders);
	static std::list<std::string> getPublicAndOtpFolders(std::string& publicFolder);
	static std::vector<std::string> getPublicAndOtpFoldersVector(std::string& publicFolder);

	
	
	static void checkGivenFolderExist(const std::list<std::string>& folders, const int mynum);
	static void checkFoldersExist(const std::list<std::string>& folders);
	static void removeFolderConfig();

	static void configureName(const std::string& name);
	static std::string getConfiguredName(bool& hasNameConf);
	static void deleteName();
	
	
private:
	static std::string getConfigPath();
	static bool configDirExist();
	static std::string getOrCreateTheConfigDir();


	static constexpr const char* nameFilename = "name.conf";
};


#endif


