#ifndef PUB_XMR_ADDRESS_H
#define PUB_XMR_ADDRESS_H


#include <string>
#include "bigint.h"

#include "xmrPubKey.h"


class PubXmrAddress
{
public:
	explicit PubXmrAddress(const std::string& addr);
	PubXmrAddress(const BigInt<257>& inPubSpend, const BigInt<257>& inPubView, const bool inIsSubaddress);
	PubXmrAddress(const XmrPubKey& inPubSpend, const XmrPubKey& inPubView, const bool inIsSubaddress = false);
	PubXmrAddress(const PubXmrAddress& other, const BigInt<65>& inPaymentId);
	PubXmrAddress(const PubXmrAddress& other, const bool dummyNoPaymentId);

	
	std::string toString() const;
	
	BigInt<257> getPubSpend() const { return pubSpendVal; }
	BigInt<257> getPubView() const { return pubViewVal; }
	
	XmrPubKey getSpendKey() const;	//Todo: Rename to getSpendPoint????
	XmrPubKey getViewKey() const;	//Todo: Rename to getViewPoint????
	
	BigInt<65> getPaymentId() const { return paymentId; }
	bool hasIdPayment() const { return hasPaymentId; }
	
	
	bool isSubAddr() const
	{
		return isSubaddress;
	}
		
private:
	std::string getPaymentval() const;

	bool isSubaddress;
	BigInt<257> pubSpendVal;
	BigInt<257> pubViewVal;

	bool hasPaymentId = false;
	BigInt<65> paymentId;
		
};



#endif

