#ifndef XMR_EC_POINT_H
#define XMR_EC_POINT_H

/**
 * ECPoint.h - Bitcoin poscript generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "bigint.h"

class XmrEllipticCurve;

class XmrECPoint
{
	friend class XmrEllipticCurve;

private:	
	XmrECPoint(const BigInt<2048>& inX, const BigInt<2048>& inY) : x(inX), y(inY) {}

public:
	
	std::string toString() const;

	//Todo: Rename? Identity?
	bool isZero() const
	{
		const BigInt<2048> zero;
		const BigInt<2048> one(1);
		
		return (x == zero && y == one);
	}
		

	const BigInt<2048>& getX() const { return x; }
	const BigInt<2048>& getY() const { return y; }

private:
	BigInt<2048> x;
	BigInt<2048> y;
};

bool operator==(const XmrECPoint& a, const XmrECPoint& b);
bool operator!=(const XmrECPoint& a, const XmrECPoint& b);


std::ostream& operator<<(std::ostream& s, const XmrECPoint& point);



#endif

