#ifndef BITGEN_XMR_PRIV_KEY_H
#define BITGEN_XMR_PRIV_KEY_H

/**
 * privKey.cpp - Bitcoin poscript generator
 *
 * Copyright (C)2015 The bitgen developers <bitgen@openmailbox.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */


#include "xmrPubKey.h"
#include "pubXmrAddress.h"
#include "xmrEC.h"



class XmrPrivKey
{
protected:
	explicit XmrPrivKey(const BigInt<256>& val, const bool inSpendKey) : key(val), isSpendKey(inSpendKey)
	{
		//Todo: Check valid range?
	}

public:
	
	XmrECPoint point(const XmrEllipticCurve& ec) const;
	XmrPubKey getPubkey(const XmrEllipticCurve& ec) const;

	std::string toString() const;

	std::string toBinaryXmrString() const;


	BigInt<257> getKey() const
	{
		return BigInt<257>::asPositive(key);
	}

	
	



	

	//Todo: private
	const BigInt<256> key;
	
protected:
	const bool isSpendKey;

};


class XmrPrivViewKey : public XmrPrivKey
{
public:
	
	explicit XmrPrivViewKey(const BigInt<256>& val) : XmrPrivKey(val, false) {}

	PubXmrAddress getSubAddressFromViewKey(const XmrPubKey& pubSpendKey, const XmrEllipticCurve& ec, const int account, const int index) const;

	PubXmrAddress getStandardAddressView(const XmrPubKey& pubKeySpend) const;
	
};


class XmrPrivSpendKey : public XmrPrivKey
{
public:
	explicit XmrPrivSpendKey(const BigInt<256>& val) : XmrPrivKey(val, true) {}

	XmrPrivViewKey privateSpendToView() const;

	PubXmrAddress getSubAddress(const XmrEllipticCurve& ec, const int account, const int index) const;

	PubXmrAddress getStandardAddress() const;

};




std::ostream& operator<<(std::ostream& str, const XmrPrivKey& k);



#endif
